@@ -1,7 +1,10 @@
-use reqwest::header::{HeaderMap, HeaderName, HeaderValue, CONTENT_TYPE, USER_AGENT};
+use http::header::{HeaderMap, HeaderName, HeaderValue, CONTENT_TYPE, USER_AGENT};
 use url::Url;
 
-use dxr_shared::{DxrError, FaultResponse, FromDXR, MethodCall, MethodResponse, ToParams};
+use crate::error::DxrError;
+use crate::fault::Fault;
+use crate::traits::{FromDXR, ToParams};
+use crate::values::{FaultResponse, MethodCall, MethodResponse};
 
 mod call;
 pub use call::*;
@@ -10,7 +13,7 @@ pub use call::*;
 #[cfg_attr(docsrs, doc(cfg(feature = "client")))]
 pub const DEFAULT_USER_AGENT: &str = concat!("dxr-client-v", env!("CARGO_PKG_VERSION"));
 
-/// builder that takes parameters for constructing a [`Client`]
+/// builder that takes parameters for constructing a [`Client`] based on [`reqwest`]
 #[derive(Debug)]
 #[cfg_attr(docsrs, doc(cfg(feature = "client")))]
 pub struct ClientBuilder {
@@ -84,12 +87,20 @@ fn request_to_body(call: &MethodCall) -> Result<String, DxrError> {
     Ok(body)
 }
 
-fn request_to_result(contents: &str) -> Result<MethodResponse, DxrError> {
+fn response_to_result(contents: &str) -> Result<MethodResponse, anyhow::Error> {
     // need to check for FaultResponse first:
     // - a missing <params> tag is ambiguous (can be either an empty response, or a fault response)
     // - a present <fault> tag is unambiguous
     let error2 = match quick_xml::de::from_str(contents) {
-        Ok(fault) => return Err(DxrError::server_fault(FaultResponse::into(fault))),
+        Ok(fault) => {
+            let response: FaultResponse = fault;
+            return match Fault::try_from(response) {
+                // server fault: return Fault
+                Ok(fault) => Err(fault.into()),
+                // malformed server fault: return DxrError
+                Err(error) => Err(error.into()),
+            };
+        },
         Err(error) => error.to_string(),
     };
 
@@ -98,16 +109,18 @@ fn request_to_result(contents: &str) -> Result<MethodResponse, DxrError> {
         Err(error) => error.to_string(),
     };
 
+    // log errors if the contents could not be deserialized as either response or fault
     log::debug!("Failed to deserialize response as either value or fault.");
     log::debug!("Response failed with: {}; Fault failed with: {}", error1, error2);
 
-    Err(DxrError::invalid_data(contents.to_owned()))
+    // malformed response: return DxrError::InvalidData
+    Err(DxrError::invalid_data(contents.to_owned()).into())
 }
 
 /// # XML-RPC client implementation
 ///
-/// This type provides a very simple XML-RPC client implementation. Initialize the [`Client`],
-/// submit a [`Call`], get a result (or a fault).
+/// This type provides a very simple XML-RPC client implementation based on [`reqwest`]. Initialize
+/// the [`Client`], submit a [`Call`], get a result (or a fault).
 #[derive(Debug)]
 #[cfg_attr(docsrs, doc(cfg(feature = "client")))]
 pub struct Client {
@@ -122,40 +135,34 @@ impl Client {
 
     /// asynchronous method for handling remote procedure calls with XML-RPC
     ///
-    /// Fault responses from the XML-RPC server are transparently converted into
-    /// [`DxrError::ServerFault`] errors.
-    pub async fn call<P: ToParams, R: FromDXR>(&self, call: Call<'_, P, R>) -> Result<R, DxrError> {
+    /// Fault responses from the XML-RPC server are transparently converted into [`Fault`] errors.
+    /// Invalid XML-RPC responses or faults will result in an appropriate [`DxrError`].
+    pub async fn call<P: ToParams, R: FromDXR>(&self, call: Call<'_, P, R>) -> Result<R, anyhow::Error> {
+        // serialize XML-RPC method call
         let request = call.as_xml_rpc()?;
-
-        // construct HTTP body and content-length header from request
         let body = request_to_body(&request)?;
 
         // construct request and send to server
-        let request = self
-            .client()
-            .post(self.url.clone())
-            .body(body)
-            .build()
-            .expect("Failed to construct POST request.");
+        let request = self.client().post(self.url.clone()).body(body).build()?;
+        let response = self.client().execute(request).await?;
 
-        let response = self.client().execute(request).await.expect("Network request failed.");
-
-        // deserialize xml-rpc method response
-        let contents = response.text().await.expect("Failed to decode response body.");
-        let result = request_to_result(&contents)?;
+        // deserialize XML-RPC method response
+        let contents = response.text().await?;
+        let result = response_to_result(&contents)?;
 
+        // extract return value  (if it is present)
         if let Some(value) = result.inner() {
-            R::from_dxr(&value)
+            Ok(R::from_dxr(&value)?)
         } else {
             #[cfg(feature = "nil")]
             {
-                use dxr_shared::Value;
-                R::from_dxr(&Value::nil())
+                use crate::values::Value;
+                Ok(R::from_dxr(&Value::nil())?)
             }
 
             #[cfg(not(feature = "nil"))]
             {
-                Err(DxrError::parameter_mismatch(0, 1))
+                Err(DxrError::parameter_mismatch(0, 1).into())
             }
         }
     }
